%% CONTROL_A_R: Perform FA controlling Acquiescence
%
% ARGUMENTS   : R   -> Correlation between item scores (The first columns as related to Social Desirability (SD)
%               N   -> Sample size
%               r   -> Number of expected number of content factors 
%               G   -> Vector with two numbers: the number of items related to SD and the number of items related to content 
%               ib  -> Balanced set of content items
%               iub -> Unbalanced set of content items (items excluded from the balanced core)
%
% RETURN VALUE: P     -> Pattern matrix
%               EV    -> Proportion of explained variance
%               GFI   -> Index of fit
%               AGFI  -> Index of fit
%               RMSR  -> Root Mean Square of Residuals
%               KELLY -> Threshold for RMSR
%               Re    -> Residual Matrix once sustracting AC
%               Rpf2  -> Final Residual Matrix
%
% DEPENDENCIES: ed.m
%               glb.m
%               mrfa.m
%               mrfan.m
%               nrm.m
%               tr.m
%
% Last update: David Navarro-Gonzalez 11/09/2017
%

function [P,EV,GFI,AGFI,RMSR,kelly,Re2,Rpf2]=control_a_r(R,N,r,G,ib,iub);

ib=ib-G(1);
iub=iub-G(1);

[m,m]=size(R);

[Af,Rf]=mrfan(R,1,10);
dRr=diag(Rf);

%% REDUCED MATRIX
Rr = R - diag(diag(R)) + diag(dRr);

Rae = Rr(G(1)+1:G(1)+G(2),G(1)+1:G(1)+G(2));

%% BALANCED CORE

%ib=[5 7 8 11 12 14 16 17 18 19 20 21 22 23 24 25]-G(1);
%ib=[6:21];
[tmp,m_b] = size(ib);
Rae_b = Rae(ib,ib);

%% CENTROID COMPUTING USING THE BALANCED CORE (TEN BERGE'S METHOD)
u=ones(m_b,1);
a= Rae_b*u/(u'*Rae_b*u)^(1/2);

%% ACQUIESCENCE FACTOR

a_g=zeros(m_b,1);
%iub=[1:5];
[m_k,tmp] = size(iub');
a_g(ib)=a;
for i=1:m_k,
    a_g(iub(i),1)=sum(Rae(iub(i),ib)')/(u'*Rae_b*u)^(1/2);
end;

Rr2 = a_g*a_g';

%Rae: MATRIX WITHOUT ACQUIESCENCE
Re = Rae-Rr2;

% NEW REDUCED MATRIX WITHOUT SD AND AC FOR CONTENT ITEMS
Re2 = R - a_g * a_g';

%% CONTENT FACTORS
[V,L]=ed(Re);
VV=V(:,1:r);
LL=L(1:r,1:r);
A=VV*sqrt(LL);
 
% FINAL P MATRIX
P = [a_g A];

% PROPORTION OF EXPLAINED VARIANCE
[Af,Rf]=mrfan(R,r+2,10);
dRr=diag(Rf);
EV = diag(P'*P)/sum(dRr);

% FINAL PRODUCED MATRIX
Rpf =P*P';
Rpf2=(P*P') + eye(size(R)) - diag(diag(P*P'));

% FINAL REDUCED MATRIX
Ref=R-Rpf;

%% Indices

%GFI  
RES = R - Rpf;
RES = RES - diag(diag(RES));
R0 = R - diag(diag(R));
GFI = 1- trace(RES*RES)/trace(R0*R0);
% GFI = trace(vec(Rpf)*vec(Rpf)')/trace(vec(R)*vec(R)');

%AGFI
k=3;
p=m;
Dk = (1.0/2.0 * ((p-k) * (p-k+1.0)) - p);
AGFI = 1.0 - ((1.0 - GFI)*( (p*(p-1.0)/2.0)/Dk));

%RMSR
A=zeros(m*(m-1)/2,1);
h = 1;
  for i=1:m,
	 for j=i+1:m,
         A(h)= Ref(j,i);
         h=h+1;
     end;
 end;
 
me=mean(A);
s=std(A);
RMSR = sqrt(s*s+me*me);
kelly = sqrt(mean(diag(R)) /(N-1));
