%% MEFA : Exploratory Factor Analysis for Multiple imputated datasets
%
% ARGUMENTS   : Xm   -> data with imputate values (N x m x k)
%               r    -> number of factors to retain
%               corr -> Correlation matrix to be used
%                           1: Variance
%                           2: Pearson
%                           3: Tetrachoric/polychoric correlation
%               rot  -> Rotation to be used
%                           %ORIGINAL               %THIS MODIFIED VERSION:
%                           0: No rotate            0: No rotate
%                           1: Consensus varimax    1: Consensus varimax
%                           2: Consensus oblimin    2: Consensus Promin
%                           3: Consensus Promin
%               sco  -> Compute factor scores:
%                           0: Don't compute (faster)
%                           1: Compute factor scores (slower, specially using polychoric correlation) 
%               H    -> Theoretical target matrix (optional)
%
% RETURN VALUE: P    -> Consensus pattern matrix (m x r)
%               PHI  -> Consensus inter-factor correlation matrix (r x r)
%               F    -> Consensus factor scores (N x r)
%               Fint -> Consensus factor scores: minimum and maximum score (N x 2 x r)
%               F_li -> Lower limit of F
%               F_ls -> Upper limit of F
%               Se   -> Standard Errors
%               reli -> Reliabilities
%
% DEPENDENCIES: list of 38 functions, for viewing all type:
%                   matlab.codetools.requiredFilesAndProducts('mefa.m')
%

function [P,PHI,F,Fint,F_li,F_ls,Se,reli]=mefa(Xm,r,corr,rot,sco,H)

[N,m,k]=size(Xm);

conv = 0.00001;

mc=0;
for ik=1:k,
    if max(max(Xm(:,:,ik))') > mc,
        mc = max(max(Xm(:,:,ik))');
    end;
end;

Am=[];Rm=zeros(m,m,k);THRESm=zeros(mc,m,k);
if corr==3
    if m<=30
        estimated_time='less than 5 minutes';
    elseif m>30 && m<=50
        estimated_time='between 5 and 10 minutes';
    elseif m>50 && m<=80
        estimated_time='between 10 and 20 minutes';
    elseif m>80 && m<=120
        estimated_time='between 20 and 40 minutes';
    elseif m>=120
        estimated_time='more than an hour';
    end;
    estimation={};
    estimation{1}=sprintf('Computing MEFA ... Estimated time remaining = %s',estimated_time);
    estimation{2}='(Highly dependent on the CPU speed)';
    BarCount = 0; bar1 = waitbar(BarCount,estimation);
    pos_1=get(bar1,'position');
    pos_2=[pos_1(1) pos_1(2)+(1.5*pos_1(4)) pos_1(3) pos_1(4)];
    set(bar1,'position',pos_2,'doublebuffer','on');
else
    BarCount = 0; bar1 = waitbar(BarCount,'Computing MEFA ... Please wait');
end;
TicPrint = tic;
TotalLines=k;
for ik=1:k,
    %% correlation matrix
    switch corr
        case 1,     %Variance
            Ri=cov(Xm(:,:,ik));
        case 2,     %Pearson
            Ri=corrcoef(Xm(:,:,ik));
        case 3,     %Polychoric
            [Ri,THRESi]=polychomap(Xm(:,:,ik),min(Xm(:,:,ik)),max(Xm(:,:,ik)),3,0,1);
            THRESm(:,:,ik) = [THRESi'; zeros(1,m)];
        otherwise,
            Ri=corrcoef(Xm(:,:,ik));
    end;
    Rm(:,:,ik)=Ri;
   

    tmp=sprintf('Correlation matrix %.0f done!',ik);disp(tmp);
    
    %%unrotated loading matrices
    Ai=my_minres(Ri,r,conv,N);
    
    Am=[Am Ai];
        
    tmp=sprintf('Loading matrix %.0f done!',ik);disp(tmp);
    
    BarCount = BarCount + 1; 
    compT = toc(TicPrint);
    compT = compT*(TotalLines-BarCount)/BarCount;
    if ik==k
        s_tmp={};
        s_tmp{1}=sprintf('Computing MEFA ... Actual imputation =  %.0f  / %.0f ) ',ik,k);
        s_tmp{2}=sprintf('(Remaining time =  %s )',Time2String( compT ));
        waitbar(BarCount/TotalLines,bar1,s_tmp);
    else
        s_tmp={};
        s_tmp{1}=sprintf('Computing MEFA ... Actual imputation =  %.0f  / %.0f ) ',ik+1,k);
        s_tmp{2}=sprintf('(Remaining time =  %s )',Time2String( compT ));
        waitbar(BarCount/TotalLines,bar1,s_tmp);
    end;
    
end;

%Consensus rotation
rm=ones(1,k)*r;
switch rot
    case 0,
        PHI = eye(r);
        [Tco,P] = cunrotated(Am,k,rm,conv,10);
    case 1,
        PHI = eye(r);
        [Tco,P] = cvarim(Am,k,rm,conv,10);
%     case 2,
%         [Tco,PHI,P] = coblimin(Am,k,rm,0,conv,10);
    case 2
        [Tco,PHI,P] = cpromin(Am,k,rm,conv,10);
    otherwise,
        PHI = eye(r);
        [Tco,P] = cunrotated(Am,k,rm,conv,10);
end;

%CONSENSUS PATTERN MATRIX (CENTROID SOLUTION)
if nargin==6,
    switch rot
        case 3
            [P, PHI] = tarrot_obl (P, abs(H), 0.001);
            [P,PHI]=orrown(H,P,PHI);
        case 4
            PHI=eye(r);
            P = tarrotorth (P, abs(H), 0.001);
            P = orrown(H,P,PHI);
    end;
end;

close(bar1);
disp('Consensus rotation done!');

% FACTOR SCORES FOR EACH SAMPLE 
Fm=zeros(N,r,k);
Fm_ls=zeros(N,r,k);
Fm_li=zeros(N,r,k);
if corr==3
    Sem=zeros(N,r,k);
    relim=zeros(N,r,k);
    Se=zeros(N,r);
    reli=zeros(N,r);
else
    Sem=zeros(r,r,k);
    relim=zeros(r,1,k);
    Se=zeros(r,r);
    reli=zeros(r,1);
end;
F=zeros(N,r);
F_li=zeros(N,r);
F_ls=zeros(N,r);


Fint=zeros(N,2,r);

%%%% FACTOR SCORES AS AVERAGE
% save tmp

if sco==1
    if corr==3 %GLOBAL WAITBAR FOR COMPUTING FACTOR SCORES
        tmp={};
        tmp{1}=('Computing factor scores, this operation can take a while, depending of');
        tmp{2}=sprintf('the number of items and imputations (5 by default) ... (Actual imputation = 1 / %.0f  )',k);
        BarCount = 0;
        bar0 = waitbar(BarCount,tmp);
        pos_1=get(bar0,'position');
        pos_2=[pos_1(1) pos_1(2)+(1.5*pos_1(4)) pos_1(3) pos_1(4)];
        set(bar0,'position',pos_2,'doublebuffer','on');
        TicPrint = tic;
        TotalLines=k;
    end;
    for ik=1:k,

        Pi = part(Am,ik,r)*part(Tco,ik,r);
        [Pi,tmp]=orrown(P,Pi,eye(r));
        switch corr
            case 1,     %Variance
                [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pi, PHI,Rm(:,:,ik));
            case 2,     %Pearson
                [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pi, PHI,Rm(:,:,ik));
            case 3,     %Polychoric
                
                %check if graded model is selected
                data=getappdata(0,'data');
                
                if data.graded==1
                     n_nodes=data.n_nodes;
                
                     Bi=THRESm(:,:,ik);
                     sigj_i = sqrt(ones(m,1)-diag(Pi*PHI*Pi'));
                     [Fi,Fi_li,Fi_ls,Se2,reli2]   = reap_grad_obli(Xm(:,:,ik), Pi, PHI, Bi, sigj_i,n_nodes,1);
                 
                else
                    %lineal model using the polychoric correlations
                    [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pi, PHI,Rm(:,:,ik));
                 
                end;
            otherwise,
                 [Fi,Fi_li,Fi_ls,Se2,reli2] = eap_continuous_obli(Xm(:,:,ik), Pi, PHI,Rm(:,:,ik));
        end;
        Fm(:,:,ik)=Fi;
        Fm_li(:,:,ik)=Fi_li;
        Fm_ls(:,:,ik)=Fi_ls;
        Sem(:,:,ik)=Se2;
        relim(:,:,ik)=reli2;

        tmp=sprintf('Factor scores matrix %.0f done!',ik);disp(tmp);
        if corr==3
            if ik==k
                close(bar0);
            else
                BarCount = BarCount + 1; 
                compT = toc(TicPrint);
                compT = compT*(TotalLines-BarCount)/BarCount;
                s_tmp={};
                s_tmp{1}=sprintf('Computing factor scores ... Actual imputation =  %.0f  / %.0f ) ',ik+1,k);
                s_tmp{2}=sprintf('(Remaining time =  %s )',Time2String( compT ));
                waitbar(BarCount/TotalLines,bar0,s_tmp);
            end;
        end;
    end;
    
    for ik=1:k,
        F= F + Fm(:,:,ik);
        F_li= F_li + Fm_li(:,:,ik);
        F_ls= F_ls + Fm_ls(:,:,ik);
        Se= Se + Sem(:,:,ik);
        reli= reli + relim(:,:,ik);
    end;

    F = F / k;
    F = F*10+50;
    
    F_li = F_li / k;
    F_li = F_li*10+50;
    
    F_ls = F_ls / k;
    F_ls = F_ls*10+50;
    
    Se = Se / k;
    reli = reli / k;

    Fint=zeros(N,2,r);
    for i=1:r,
        tmp=[];
        for ik=1:k,
            tmp=[tmp Fm(:,i,ik)];
        end;
        Fint(:,1,i)=min(tmp')';
        Fint(:,2,i)=max(tmp')';
    end;
end;

